use bevy::{prelude::*, render::view::RenderLayers};
use bevy::color::palettes::css::{GREEN, RED};

const LAYER_A: usize = 0;
const LAYER_B: usize = 1;

fn main() {
  App::new()
    .add_plugins(DefaultPlugins)
    .add_systems(Startup, setup)
    .run();
}

// Bug - when using two cameras, where one has HDR and the other one doesn't, one of the cameras will not render anything. 
// Which one fails to render depends on render order. 
//   - In the example below, LAYER_A renders first and has hdr: false. LAYER_B has hdr. Layer A will not be visible.
//   - If you swap the render order of the cameras, LAYER_B will not be visible.
//   - If you set hdr: true on both cameras, both layers will render fine.
// 

fn setup(
  mut commands: Commands,
  mut meshes: ResMut<Assets<Mesh>>,
  mut standard_materials: ResMut<Assets<StandardMaterial>>,
) {
  commands.spawn((
    Camera3dBundle {
      transform: Transform::from_translation(Vec3::splat(10.0)).looking_at(Vec3::ZERO, Vec3::Y),
      camera: Camera {
        clear_color: ClearColorConfig::Default,
        order: 0,
        // Setting hdr to true on this camera will make both cameras render fine.
        // hdr: true,
        ..Default::default()
      },
      ..Default::default()
    }, 
    RenderLayers::from_layers(&[LAYER_A])
  ));

  commands.spawn((
    Camera3dBundle {
      transform: Transform::from_translation(Vec3::splat(10.0)).looking_at(Vec3::ZERO, Vec3::Y),
      camera: Camera {
        clear_color: ClearColorConfig::None,
        order: 1,
        hdr: true,
        ..Default::default()
      },
      ..Default::default()
    }, 
    RenderLayers::from_layers(&[LAYER_B])
  ));

  commands.insert_resource(AmbientLight {
    color: Color::WHITE,
    brightness: 1000.0,
  });

  commands.spawn(DirectionalLightBundle {
    transform: Transform::from_xyz(-50.0, 50.0, 50.0).looking_at(Vec3::ZERO, Vec3::Y),
    directional_light: DirectionalLight {
      illuminance: 3000.,
      ..default()
    },
    ..default()
  });

  commands.spawn(MaterialMeshBundle {
    transform: Transform::from_translation(Vec3::new(-2.0, 0.0, 0.0)),
    mesh: meshes.add(Cuboid::from_size(Vec3::splat(2.0))),
    material: standard_materials.add(Color::from(RED)),
    ..default()
  }).insert(RenderLayers::from_layers(&[LAYER_A]));

  commands.spawn(MaterialMeshBundle {
    transform: Transform::from_translation(Vec3::new(2.0, 0.0, 0.0)),
    mesh: meshes.add(Cuboid::from_size(Vec3::splat(2.0))),
    material: standard_materials.add(Color::from(GREEN)),
    ..default()
  }).insert(RenderLayers::from_layers(&[LAYER_B]));
}
